﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using JsonRpcLite.Services;
using MailKit.Net.Smtp;
using MimeKit;
using Newtonsoft.Json.Linq;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Service;

namespace WingEmailModule.Service
{
    public class EmailService : JsonRpcService, IEmailService
    {    
        private string _serverHost;
        private string _sender;
        private string _password;   
        private int _port;
        private bool _useSSL;
        /// <summary>
        /// Init service
        /// </summary>
        public EmailService()
        {           
            _serverHost = ConfigurationManager.GetParammeter<StringParameter>("Email", "MailHost").Value;
            _sender = ConfigurationManager.GetParammeter<StringParameter>("Email", "EmailUserName").Value;
            _password = ConfigurationManager.GetParammeter<StringParameter>("Email", "EmailUserPassword").Value;
            _port = ConfigurationManager.GetParammeter<IntParameter>("Email", "SmtpPort").Value;
            _useSSL = ConfigurationManager.GetParammeter<BoolParameter>("Email", "UseSSL").Value;
        }

        /// <summary>
        /// 通用异常处理方法
        /// </summary>
        /// <param name="customerRpcCodeEnum">异常标识码</param>
        /// <param name="msg">异常信息</param>
        /// <param name="internalMessage">内部异常信息，非必填</param>
        private void ThrowCustomerException(CustomerRpcCode customerRpcCodeEnum, string msg, string internalMessage = null)
        {
            ThrowRpcException((int)customerRpcCodeEnum, msg, internalMessage);
        }

        /// <summary>
        /// Send email
        /// </summary>
        /// <param name="receivers"></param>
        /// <param name="templateName"></param>
        /// <param name="templateArg"></param>
        public async Task<bool> SendEmailAsync(string language, List<string> receivers, string templateName, params string[] templateArg)
        {
            var mailMessage = new MimeMessage();
            if (!string.IsNullOrEmpty(_sender) && _sender.Split('@').Length > 1) 
            {
                var fromAddress = new MailboxAddress(_sender.Split('@')[0], _sender);
                mailMessage.From.Add(fromAddress);
            }
            if (receivers != null && receivers.Count > 0)
            {
                foreach (var receiver in receivers)
                {
                    if (!string.IsNullOrEmpty(receiver) && receiver.Split('@').Length > 1) 
                    {
                        var toAddress = new MailboxAddress(receiver.Split('@')[0], receiver);
                        mailMessage.To.Add(toAddress);
                    }
                }
            }
            var jsonContent = WingServerCommon.ResourceManage.ResourceManager.GetSetting(language + "emailTemplate");
            //var jsonContent = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, $"Resource\\Languge\\{language}", "emailTemplate.json"));
            if (string.IsNullOrWhiteSpace(jsonContent))
            {
                language = "en-US";
                jsonContent = WingServerCommon.ResourceManage.ResourceManager.GetSetting(language + "emailTemplate");
            }
            JObject json = JObject.Parse(jsonContent);
            var templateSubject = json[templateName]["Subject"].ToString();
            mailMessage.Subject = templateSubject;
            var templateContent = json[templateName]["Body"].ToString();
            mailMessage.Body = new TextPart("plain") 
            { 
                Text =  string.Format(templateContent, templateArg)
            };
            var objSmtpClient = new SmtpClient { Timeout = 6000 };
            try
            {
                objSmtpClient.ServerCertificateValidationCallback = (s, c, h, e) => true;
                objSmtpClient.Connect(_serverHost, _port, true);
                objSmtpClient.AuthenticationMechanisms.Remove("XOAUTH2");
                objSmtpClient.Authenticate(_sender, _password);
                objSmtpClient.Send(mailMessage);
            }
            catch (Exception ex)
            {
                ThrowCustomerException(CustomerRpcCode.ExceptionError, $"Send emial failed {ex.InnerException?.Message}");
            }
            finally
            {
                objSmtpClient.Disconnect(true);
                objSmtpClient.Dispose();
            }
            return await Task.FromResult(true);
        }
    }
}
